#!/bin/zsh

# Created Feb 23, 2005
# (c) 2005 Simon Brown. All rights reserved.
#
# 05/02/24	Fixed problem where we were setting the current directory in
#			copyItem to the wrong directory. copyItem no longer sets
#			directory, instead psdh and pdir do.
# 05/02/25	Changed from using cp to ditto since ditto will make any needed
#			directories at the destination.
# 05/02/26	Converted `...` expressions to use command substitution for speed.
#			Added in error checking so that we don't attempt to copy files
#			from a source directory we can't find.
#			Added some sanity checking of source & destination paths.
#			Made sure we're starting in the correct directory when we start!
#			(Forgot to cd to initial source path).
#			Had to remove error message stating what wasn't copied when there
#			was a missing directory, since it was difficult
#			to determine what hadn't been copied.
# 05/02/28	Added some additional files to FCS file list.
#			Now check dpath to see if a new user is likely being setup, and then
#			chown the files to the right owner.
#			Now use the -q option with read to get y/n answers.
# 05/04/06	Added Stickies files and iTunes' pref file.
# 05/04/07	More thorough repairing permissions by setting group id and making
#			sure user has read & write permissions. Also changed wording
#			when asking whether to do this.
# 05/04/14	Can now optionally copy .../SystemConfiguration/preferences.plist
# 05/04/15	Need to save a copy of the destination path as given by user,
#			just in case all of the pshd's weren't cleaned up with popd's.
#			Fixed a problem with the network preferences save feature by
#			making it a post-copy procedure.
# 05/04/17	Discovered that you had to be root to restore networking preference
#			.plist. Add info @ this when prompting user.


dirErr=0					# Clear the directory error flag.
restoreNetPrefs="n"			# "y" to restore the saved network preferences
saveNetPrefs="n"			# "y" to save the current network preferences
savedDPath=""				# Saved copy of dpath for post-copy procedures
destDir=""					# Destination path supplied by user (dpath - base)

echo
echo "Copy User Folder"
echo " version 050417"
echo
echo
echo "BACKUP: The source folder will be the users home directory,"
echo "(e.g. /Users/jdoe). Destination folder will the location to store"
echo "a copy of user folder (e.g., a folder named jdoe will be created"
echo "in the destination folder)."
echo
echo "RESTORE: Source will be the user folder in the backup location."
echo "Destination will be the folder containing the user folder"
echo "(the destination is NOT the user folder itself)."
echo
echo "Note: You may drag a folder from a Finder window into the"
echo "Terminal window to enter the required paths."
echo
echo
echo "Source user folder: \c"
read spath
echo
echo "Destination of user folder: \c"
read destDir
echo

if [ -z "$destDir" ]; then
	echo "Error! Destination path is empty"
	echo "Script halted"
	return 1
fi

if [ -z "$spath" ]; then
	echo "Error! Source path is empty"
	echo "Script halted"
	return 1
fi

# Need to make sure that there is a trailing forward slash on path.
#spath=`echo $spath | sed -e "s/$/\//"`

sbase=$(basename $spath)

if [ -z "$sbase" ]; then
	echo "Error! Source folder is empty"
	echo "Script halted"
	return 1
fi

# Add in the user directory name to create destination path.
dpath=$destDir/$sbase

# Save this copy for any post copy procedures we may need to run.
savedDPath=$dpath

if [[ $spath == $dpath ]]; then
	echo "Error: Destination and source are the same!"
	echo "Script halting"
	return 1
fi

# Copying from one part of /Users to another?
if [[ $spath = /Users* && ( $destDir = /Users* ) ]]; then
	echo "Warning! Source and destination both in /Users."
	echo "Continue running script (y/n)?\c"
	read -q continue
	if [[ $continue != "y" ]]; then
		echo "Stopping script"
		return 1
	fi
fi

if [[ "$destDir" == "/" || ( "$destDir" == "/Volumes" ) ]]; then
	echo "Warning! You appear to be copying to a root or volume level directory."
	echo "Do you really want to do this (y/n)?\c"
	read -q continue
	if [[ $continue != "y" ]]; then
		echo "Stopping script"
		return 1
	fi
fi

if [[ $destDir == "/Users" ]]; then
	echo "You appear to be setting up a new user. Do you want "
	echo "to reset permissions for all files to the user $dbase (y/n)?\c"
	read -q chownFiles
	echo	
	echo "Do you want to restore the user's old network settings - this"
	echo "will require root permissions (y/n)?\c"
	read -q restoreNetPrefs
	echo
fi

if [[ $spath = /Users/* ]]; then
	echo "It appears that you may be saving an existing user's files. Do you want "
	echo "to save a copy of the current network settings (y/n)?\c"
	read -q saveNetPrefs
	echo
fi

#
#        c o p y I t e m
#

# Reads in file or directory names from std input, copying each
# one recursively.
#
# ON ENTRY:
# $spath = path to use for source files
# $dpath = path to use for destination files

copyItem ()
{
	# get 1st file/folder item
	read fname
	
	# repeat until read command exhuasts stdio
	
	while [ -n "$fname" ]; do
			echo ".\c"
			# Don't follow to other devices, copy resource fork
			ditto -X -rsrc "$fname" "$dpath/$fname"
			if [[ $? != 0 ]]; then
				echo
				echo "Continue copying this folder (y/n)\c?"
				read -q continue
				echo
				if [[ $continue != "y" ]]; then
					return 1
				fi
			fi
			read fname
	done
}

cpne ()
{
	if (( ! dirErr )); then

#		How can I add a $ at the end of this???

		fileMatch=$(echo $1 | sed -e "s/|/$|/g")
		ls -1 | egrep -v $fileMatch | copyItem
	fi
}

cpeq ()
{
	# If there's a problem cd'ing to the right directory skip over copy.
	if (( ! dirErr )); then
		fileMatch=$(echo $1 | sed -e "s/|/$|/g")
		ls -1 | egrep $fileMatch | copyItem
	fi
}

#
#        p s h d
#

# Descend one level from current directory of both spath & dpath to the
# directory given as a parameter.

# NOTE: this version of pshd can only descend into directories.
# popd must be used to ascend.

pshd ()
{
	# Check that parameter is a directory & it is readable.
	if [ -d $1 ]; then
		if [ -r $1 ]; then
			# $1 is both a directory & readable
			# Save our current location in source path,
			# then cd to new location.
			pushd $1
	
			# Adjust the destination path to match new source path.
			dpath=$dpath/$1
		else
			echo "\nWarning: `pwd`/$1 is not readable and will be skipped"
			# If there was an error changing source path then don't change destination
			dirErr=1
		fi
	else
		echo "\nWarning: folder `pwd`/$1 not found"
		dirErr=1
	fi

	#echo "\nspath="$(pwd) "  dpath=$dpath"
}

pdir ()
{
	# Only do pop if push was succesful.
	if (( ! dirErr )); then
	
		# Save our current location in source path,
		# then cd to new location.
		popd
		
		# Adjust the destination path to match new source path.
		dpath=$(dirname $dpath)
	else
		dirErr=0
	fi

	#echo "spath=" $(pwd) "  dpath=$dpath"
}

# Start copying from this directory.
cd $spath

###################
###  B E G I N  ###
###################

cpne "Library|Movies|Public|Sites"
pshd "Library"
cpeq "Application Support|Assistants|Audio|Calendars|ColorSync|Documentation|Favorites|FontCollections|Mail|Mozilla|Printers|Sounds|StickiesDatabase"
pshd "Safari"
cpeq "Bookmarks.plist"
pdir
pshd "Preferences"
cpeq "com.apple.dock.plist|com.apple.desktop.plist|com.apple.internetconfig.plist|com.apple.iTunes.plist|com.apple.Stickies.plist|FontBook.plist"
pshd "Explorer"
cpeq "Favorites.html"
pdir
pdir

###############
###  E N D  ###
###############

if [[ $chownFiles == "y" ]]; then
	echo "chowning files to $sbase"
	# sbase == short user name for user
	# Change the owner of all files to user.
	chown -R $sbase $savedDPath
	# In Panther, each user has its own group too.
	chgrp -R $sbase $savedDPath
	# Make the user can read and write to their files.
	# NOTE: how would you fix execute permissions here?
	chmod -R ug+rw $savedDPath
fi

if [[ $restoreNetPrefs == "y" ]]; then
	echo "\nRestoring network preferences file.\n"

	# Copy the saved file (if present) w/o asking if its OK to overwrite and
	# preserving permissions.
	cp -fpv $spath/preferences.plist /Library/Preferences/SystemConfiguration/
fi

if [[ $saveNetPrefs == "y" ]]; then
	echo "\nSaving network preferences file.\n"

	# Copy the saved file (if present) w/o asking if its OK to overwrite and
	# preserving permissions. We'll save the file in the saved copy of the
	# user's home directory.
	cp -fpv /Library/Preferences/SystemConfiguration/preferences.plist $savedDPath/ 
fi


echo "\n"

